# Example pys60 audio app for CCRMA Mobile Music Workshop, 2007-11-01
# Copyright (c) Nokia, written by Jarno Seppänen
# additions by Henri Penttinen, Stanford CCRMA / TKK Helsinki University of Technology 2007-11-04

import appuifw
import e32
import key_codes
import shake # C++ audio engine

# Setup trace output
if e32.in_emulator():
    TRACEFILE=file('c:\\pocketshake.log','a',0)
else:
    TRACEFILE=file('c:\\pocketshake.log','a',0)					# this goes on phone memory 
#   TRACEFILE=file('e:\\pocketshake.log','a',0)					# this goes on card memory 
def TRACE(txt, *args): TRACEFILE.write((txt%args)+'\n')

# name (unicode), 3x freq (Hertz), 3x range (semitones), 3x gain
PRESETS=[(u'1D unison X', [440.0, 0, 0], [24.0, 0, 0], [1.0, 0, 0]),
	   (u'1D unison Y', [0, 440.0, 0], [0, 24.0, 0], [0, 1.0, 0]),
	   (u'1D unison Z', [0, 0, 440.0], [0, 0, 24.0], [0, 0, 1.0]),
         (u'3D unison', [440.0, 440.0, 440.0], [12.0, 12.0, 12.0], [0.33, 0.33, 0.33]),
         (u'C major', [440.0*2**(3.0/12), 440.0*2**(7.0/12), 440.0*2**(10.0/12)], [12.0, 12.0, 12.0], [0.33, 0.33, 0.33]),
         (u'F major', [440.0*2**(8.0/12), 440.0*2**(12.0/12), 440.0*2**(15.0/12)], [12.0, 12.0, 12.0], [0.33, 0.33, 0.33]),
         (u'G major', [440.0*2**(10.0/12), 440.0*2**(14.0/12), 440.0*2**(17.0/12)], [12.0, 12.0, 12.0], [0.33, 0.33, 0.33]),
         (u'Borduna', [440.0*2**(3.0/12), 440.0*2**(3.0/12), 440.0*2**(10.0/12)], [12.0, 0.3, 0.3], [0.5, 0.25, 0.25]),
         (u'2D Borduna', [440.0*2**(8.0/12), 440.0*2**(12.0/12), 440.0*2**(15.0/12)], [0.2, 20.0, 0.2], [0.21, 0.5, 0.21]),
         (u'Wacky', [220.0, 220.0, 220.0], [100.0, 100.0, 100.0], [0.33, 0.33, 0.33]),
         (u'Edit...', [440.0, 440.0, 440.0], [12.0, 12.0, 12.0], [0.33, 0.33, 0.33])]
         
class App(object):
    def __init__(self):
        TRACE("__init__()")
        self.volume=0.5
        self.editing=False        # true iff preset edit view shown
        self.freq=PRESETS[-1][1]
        self.range=PRESETS[-1][2]
        self.gain=PRESETS[-1][3]
        self.lbox_index=0
        self.shake=shake.Shake()
        self.exit_lock=e32.Ao_lock()
        appuifw.app.title=u"PocketShake"
        appuifw.app.menu=[(u'Exit', self.exit_lock.signal)]
        appuifw.app.exit_key_handler=harness(self.exit_lock.signal)
        self.view=appuifw.Listbox([u''], harness(self.select))
        self.redraw()
        self.view.bind(key_codes.EKeyLeftArrow,harness(self.parameter_decr))
        self.view.bind(key_codes.EKeyRightArrow,harness(self.parameter_incr))
        self.view.bind(key_codes.EKeyUpArrow,harness(self.prev_preset))
        self.view.bind(key_codes.EKeyDownArrow,harness(self.next_preset))
        self.view.bind(key_codes.EKey0,harness(self.shake.Close))
        self.view.bind(key_codes.EKeyStar,harness(self.volume_down))
        self.view.bind(key_codes.EKeyHash,harness(self.volume_up))
        self.tooltip=appuifw.InfoPopup()

    def run(self):
        TRACE("run()")
        appuifw.app.body=self.view
        e32.ao_yield() # redraw
        self.shake.Open()
        self.change_preset()      # activate first preset
        self.shake.SetVolume(self.volume)
        self.exit_lock.wait()
        TRACE("run() cleaning up")
        self.shake.Close()
        appuifw.app.exit_key_handler=None
        appuifw.app.body=None
        e32.ao_yield() # redraw
        appuifw.app.set_exit()
        TRACE("run() exiting")

    def redraw(self):
        TRACE("redraw()")
        if self.editing:
            self.view.set_list([u'X freq. %d Hz'%self.freq[0],
                                u'Y freq. %d Hz'%self.freq[1],
                                u'Z freq. %d Hz'%self.freq[2],
                                u'X range %d semi'%self.range[0],
                                u'Y range %d semi'%self.range[1],
                                u'Z range %d semi'%self.range[2],
                                u'X gain %.1f'%self.gain[0],
                                u'Y gain %.1f'%self.gain[1],
                                u'Z gain %.1f'%self.gain[2]],
                               self.view.current())
        else:
            self.view.set_list([p[0] for p in PRESETS],
                               self.lbox_index)

    def select(self):
        TRACE("select()")
        if self.editing or self.view.current()==len(PRESETS)-1:
            self.editing=1-self.editing
            self.redraw()

    def prev_preset(self):
        TRACE("prev_preset()")
        if self.editing:
            return
        self.lbox_index=(self.lbox_index-1+len(PRESETS))%len(PRESETS)
        self.change_preset()
        
    def next_preset(self):
        TRACE("next_preset()")
        if self.editing:
            return
        self.lbox_index=(self.lbox_index+1)%len(PRESETS)
        self.change_preset()
        
    def change_preset(self):
        TRACE("change_preset()")
        if self.editing:
            return
        TRACE("idx=%d", self.lbox_index)
        TRACE("preset %s", PRESETS[self.lbox_index][0])
        for i in range(3):
            self.shake.SetFrequency(i, PRESETS[self.lbox_index][1][i])
            self.shake.SetPitchRange(i, PRESETS[self.lbox_index][2][i]/12.0)
            self.shake.SetGain(i, PRESETS[self.lbox_index][3][i])

    def parameter_decr(self):
        TRACE("parameter_decr()")
        if not self.editing:
            return
        idx=self.view.current()
        TRACE("idx=%d", idx)
        if idx<3:
            self.freq[idx]*=2**(-1.0/12)
            self.shake.SetFrequency(idx, self.freq[idx])
        elif idx<6:
            idx-=3
            self.range[idx]-=1
            self.shake.SetPitchRange(idx, self.range[idx]/12.0)
        else:
            idx-=6
            self.gain[idx]=max(0, self.gain[idx]-0.1)
            self.shake.SetGain(idx, self.gain[idx])
        self.redraw()

    def parameter_incr(self):
        TRACE("parameter_incr()")
        if not self.editing:
            return
        idx=self.view.current()
        TRACE("idx=%d", idx)
        if idx<3:
            self.freq[idx]*=2**(1.0/12)
            self.shake.SetFrequency(idx, self.freq[idx])
        elif idx<6:
            idx-=3
            self.range[idx]+=1
            self.shake.SetPitchRange(idx, self.range[idx]/12.0)
        else:
            idx-=6
            self.gain[idx]=min(1, self.gain[idx]+0.1)
            self.shake.SetGain(idx, self.gain[idx])
        self.redraw()
        
    def volume_up(self):
        TRACE("volume_up()")
        self.volume=min(1, self.volume+0.1)
        self.shake.SetVolume(self.volume)
        self.tooltip.show(u'Volume %f'%self.volume)

    def volume_down(self):
        TRACE("volume_down()")
        self.volume=max(0, self.volume-0.1)
        self.shake.SetVolume(self.volume)
        self.tooltip.show(u'Volume %f'%self.volume)
    
def harness(fun):
    '''Exception harness creator.
    See SF bug #1542568 "System error -50 if key callback raises"
    http://sourceforge.net/tracker/index.php?func=detail&aid=1542568&group_id=154155&atid=790646'''
    def hfun():
        try:
            return fun()
        except:
            import traceback
            TRACE("Exception harness:")
            traceback.print_exc(file=TRACEFILE)
    return hfun
    #return e32.ao_callgate(hfun) # fixes AO hang-up problems

def run():
    a=App()
    a.run()

try:
    run()
except:
    import traceback
    TRACE("uncaught exception:")
    traceback.print_exc(file=TRACEFILE)
